@@ -68,6 +68,20 @@ pub fn fmt_err(msg: &str) -> String {
     format!("{}: {}", status_colorizer("ERROR"), msg)
 }
 
+/// simple wrapper to get the current system time as
+/// time elapsed from unix epoch
+pub fn timestamp() -> f64 {
+    let since_the_epoch = SystemTime::now()
+        .duration_since(UNIX_EPOCH)
+        .unwrap_or_else(|_| Duration::from_secs(0));
+
+    let secs = since_the_epoch.as_secs() as f64;
+    let nanos = since_the_epoch.subsec_nanos() as f64;
+
+    // Convert nanoseconds to fractional seconds and add to secs
+    secs + (nanos / 1_000_000_000.0)
+}
+
 /// given a FeroxResponse, send a TryRecursion command
 ///
 /// moved to utils to allow for calls from extractor and scanner
@@ -563,6 +577,13 @@ pub fn parse_url_with_raw_path(url: &str) -> Result<Url> {
         bail!("url to parse has no authority and is therefore invalid");
     }
 
+    // thanks to @devx00: the possibility exists for Url to return true for
+    // has_authority, but not have a host/port, so we'll check for that
+    // and bail if it's the case
+    if parsed.host().is_none() {
+        bail!("url to parse doesn't have a host");
+    }
+
     // we have a valid url, the next step is to check the path and see if it's
     // something that url::Url::parse would silently transform
     //
@@ -648,7 +669,7 @@ pub fn parse_url_with_raw_path(url: &str) -> Result<Url> {
     // each of the following is a string that we can expect url::Url::parse to
     // transform. The variety is to ensure we cover most common path traversal
     // encodings
-    let transformation_detectors = vec![
+    let transformation_detectors = [
         // ascii
         "..",
         // single url encoded
@@ -729,6 +750,18 @@ mod tests {
     use crate::config::Configuration;
     use crate::scan_manager::{FeroxScans, ScanOrder};
 
+    #[test]
+    /// parse_url_with_raw_path with javascript:// should not throw an unimplemented! error
+    fn utils_parse_url_with_raw_path_javascript() {
+        let url = "javascript://";
+        let parsed = parse_url_with_raw_path(url);
+        assert!(parsed.is_err());
+        assert!(parsed
+            .unwrap_err()
+            .to_string()
+            .contains("url to parse doesn't have a host"));
+    }
+
     #[test]
     /// multiple tests for parse_url_with_raw_path
     fn utils_parse_url_with_raw_path() {
@@ -942,7 +975,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -961,7 +998,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -980,7 +1021,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1001,7 +1046,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1029,7 +1078,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1047,7 +1100,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/api/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1066,7 +1123,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/not-denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1085,7 +1146,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/stuff/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1104,7 +1169,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/api/not-denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.url_denylist = vec![Url::parse(deny_url).unwrap()];
@@ -1124,7 +1193,11 @@ mod tests {
         let tested_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.regex_denylist = vec![Regex::new(deny_pattern).unwrap()];
@@ -1145,7 +1218,11 @@ mod tests {
         let tested_https_url = Url::parse("https://testdomain.com/denied/").unwrap();
 
         let scans = Arc::new(FeroxScans::default());
-        scans.add_directory_scan(scan_url, ScanOrder::Initial);
+        scans.add_directory_scan(
+            scan_url,
+            ScanOrder::Initial,
+            Arc::new(Handles::for_testing(None, None).0),
+        );
 
         let mut config = Configuration::new().unwrap();
         config.regex_denylist = vec![Regex::new(deny_pattern).unwrap()];
